/*
(c) Janusz Ganczarski
http://www.januszg.hg.pl
JanuszG@enter.net.pl
*/

#include <GL/glut.h>
#include <GL/glext.h>
#ifndef WIN32
#define GLX_GLXEXT_LEGACY
#include <GL/glx.h>
#define wglGetProcAddress(name) glXGetProcAddressARB((GLubyte *)name)
#endif
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

// stae do obsugi menu podrcznego

enum
{
    PAGE_1,  // pierwsza strona
    PAGE_2,  // druga strona
    PAGE_3,  // trzecia strona
    EXIT     // wyjcie
};

// numer wywietlanej strony

int page_no = PAGE_1;

// funkcja rysujca napis w wybranym miejscu

void DrawString (int x, int y, char *string)
{
    // pooenie napisu
    glRasterPos2i (x,y);

    // wywietlenie napisu
    int len = strlen (string);
    for (int i = 0; i < len; i++)
        glutBitmapCharacter (GLUT_BITMAP_9_BY_15,string [i]);
}

// wywietlenie pierwszej strony ze zmiennym stanu zalenymi od implementacji

void Page1 ()
{
    // cig znakw uywany przy wywietlaniu napisw
    char string [256];

    // pobierane wartoci
    GLint ivar,itab [2];
    GLboolean bval;
    GLfloat fval,ftab [2];

    // wysoko okna renderingu
    GLuint height = glutGet (GLUT_WINDOW_HEIGHT) - 15;

    // maksymalna ilo wiate
    glGetIntegerv (GL_MAX_LIGHTS,&ivar);
    sprintf (string,"GL_MAX_LIGHTS: %i (8)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // maksymalna ilo paszczyn obcinania
    glGetIntegerv (GL_MAX_CLIP_PLANES,&ivar);
    sprintf (string,"GL_MAX_CLIP_PLANES: %i (6)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // maksymalna gboko stosu macierzy koloru
    // SGI_color_matrix, ARB_imaging_subset
    glGetIntegerv (GL_MAX_COLOR_MATRIX_STACK_DEPTH,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_COLOR_MATRIX_STACK_DEPTH: %i (2)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalna gboko stosu macierzy modelowania
    glGetIntegerv (GL_MAX_MODELVIEW_STACK_DEPTH,&ivar);
    sprintf (string,"GL_MAX_MODELVIEW_STACK_DEPTH: %i (32)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // maksymalna gboko stosu macierzy modelowania
    glGetIntegerv (GL_MAX_PROJECTION_STACK_DEPTH,&ivar);
    sprintf (string,"GL_MAX_PROJECTION_STACK_DEPTH: %i (2)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // maksymalna gboko stosu macierzy tekstury
    glGetIntegerv (GL_MAX_TEXTURE_STACK_DEPTH,&ivar);
    sprintf (string,"GL_MAX_TEXTURE_STACK_DEPTH: %i (2)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // ilo bitw precyzji podpikseli ekranu
    glGetIntegerv (GL_SUBPIXEL_BITS,&ivar);
    sprintf (string,"GL_SUBPIXEL_BITS: %i (4)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // maksymalny rozmiar tekstury trjwymiarowej
    // OpenGL 1.2, EXT_texture3D
    glGetIntegerv (GL_MAX_3D_TEXTURE_SIZE,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_3D_TEXTURE_SIZE: %i (16)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalny rozmiar tekstury jedno i dwuwymiarowej
    glGetIntegerv (GL_MAX_TEXTURE_SIZE,&ivar);
    sprintf (string,"GL_MAX_TEXTURE_SIZE: %i (64)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // modu maksymalnego przesunicia poziomu  szczegowoci (LOD) mipmap
    glGetIntegerv (GL_MAX_TEXTURE_LOD_BIAS,&ivar);
    // OpenGL 1.4, EXT_texture_lod_bias
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_TEXTURE_LOD_BIAS: %i (2)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalny rozmiar teksury szeciennej
    // OpenGL 1.3, ARB_texture_cube_map, EXT_texture_cube_map
    glGetIntegerv (GL_MAX_CUBE_MAP_TEXTURE_SIZE,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_CUBE_MAP_TEXTURE_SIZE: %i (16)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalna ilo elementw tablicy przeksztace mapy pikselowej
    glGetIntegerv (GL_MAX_PIXEL_MAP_TABLE,&ivar);
    sprintf (string,"GL_MAX_PIXEL_MAP_TABLE: %i (32)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // maksymalna gboko stosu nazw
    glGetIntegerv (GL_MAX_NAME_STACK_DEPTH,&ivar);
    sprintf (string,"GL_MAX_NAME_STACK_DEPTH: %i (64)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // maksymalne zagniedenie wywoa list wywietlania
    glGetIntegerv (GL_MAX_LIST_NESTING,&ivar);
    sprintf (string,"GL_MAX_LIST_NESTING: %i (64)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // maksymalna ilo punktw kontrolnych krzywej
    glGetIntegerv (GL_MAX_EVAL_ORDER,&ivar);
    sprintf (string,"GL_MAX_EVAL_ORDER: %i (8)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // maksymalna wielko obszaru renderingu
    glGetIntegerv (GL_MAX_VIEWPORT_DIMS,itab);
    sprintf (string,"GL_MAX_VIEWPORT_DIMS: %i %i (%i %i)",itab[0],itab[1],
             glutGet (GLUT_SCREEN_WIDTH),glutGet (GLUT_SCREEN_HEIGHT));
    DrawString (0,height,string);
    height -= 16;

    // maksymalna gboko stosu artubutw po stronie serwera
    glGetIntegerv (GL_MAX_ATTRIB_STACK_DEPTH,&ivar);
    sprintf (string,"GL_MAX_ATTRIB_STACK_DEPTH: %i (16)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // maksymalna gboko stosu artubutw po stronie klienta
    glGetIntegerv (GL_MAX_CLIENT_ATTRIB_STACK_DEPTH,&ivar);
    sprintf (string,"GL_MAX_CLIENT_ATTRIB_STACK_DEPTH: %i (16)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // ilo pomocniczych buforw koloru
    glGetIntegerv (GL_AUX_BUFFERS,&ivar);
    sprintf (string,"GL_AUX_BUFFERS: %i (0)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // warto GL_TRUE, gdy bufor kolorw pracuje w trybie RGB(A)
    glGetBooleanv (GL_RGBA_MODE,&bval);
    if (bval == GL_TRUE)
        DrawString (0,height,"GL_RGBA_MODE: GL_TRUE");
    else
        DrawString (0,height,"GL_RGBA_MODE: GL_FALSE");
    height -= 16;

    // warto GL_TRUE, gdy bufor kolorw pracuje w trybie indeksowym
    glGetBooleanv (GL_INDEX_MODE,&bval);
    if (bval == GL_TRUE)
        DrawString (0,height,"GL_INDEX_MODE: GL_TRUE");
    else
        DrawString (0,height,"GL_INDEX_MODE: GL_FALSE");
    height -= 16;

    // warto GL_TRUE, gdy bufor koloru jest podwjnie buforowany
    glGetBooleanv (GL_DOUBLEBUFFER,&bval);
    if (bval == GL_TRUE)
        DrawString (0,height,"GL_DOUBLEBUFFER: GL_TRUE");
    else
        DrawString (0,height,"GL_DOUBLEBUFFER: GL_FALSE");
    height -= 16;

    // warto GL_TRUE, gdy bufor koloru pracuje w trybie stereoskopowym
    glGetBooleanv (GL_STEREO,&bval);
    if (bval == GL_TRUE)
        DrawString (0,height,"GL_STEREO: GL_TRUE");
    else
        DrawString (0,height,"GL_STEREO: GL_FALSE");
    height -= 16;

    // zakres wielkoci punktw bez antyaliasingiu
    // OpenGL 1.2
    glGetFloatv (GL_ALIASED_POINT_SIZE_RANGE,ftab);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_ALIASED_POINT_SIZE_RANGE: %f %f (1.0 1.0)",ftab[0],ftab[1]);
        DrawString (0,height,string);
        height -= 16;
    }

    // zakres wielkoci punktw z antyaliasingiem
    // OpenGL 1.2, OpenGL 1.0 staa: GL_POINT_SIZE_RANGE
    glGetFloatv (GL_SMOOTH_POINT_SIZE_RANGE,ftab);
    sprintf (string,"GL_SMOOTH_POINT_SIZE_RANGE: %f %f (1.0 1.0)",ftab[0],ftab[1]);
    DrawString (0,height,string);
    height -= 16;

    // krok zmiany wielkoci punktw z antyaliasingiem
    // OpenGL 1.2
    glGetFloatv (GL_SMOOTH_POINT_SIZE_GRANULARITY,&fval);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_SMOOTH_POINT_SIZE_GRANULARITY: %f",fval);
        DrawString (0,height,string);
        height -= 16;
    }

    // minimalna i maksymalna szeroko linii bez antyaliasinu
    // OpenGL 1.2
    glGetFloatv (GL_ALIASED_LINE_WIDTH_RANGE,ftab);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_ALIASED_LINE_WIDTH_RANGE: %f %f (1.0 1.0)",ftab[0],ftab[1]);
        DrawString (0,height,string);
        height -= 16;
    }

    // minimalna i maksymalna szeroko linii z antyaliasingiem
    // OpenGL 1.2, OpenGL 1.0 staa: GL_LINE_WIDTH_RANGE
    glGetFloatv (GL_SMOOTH_LINE_WIDTH_RANGE,ftab);
    sprintf (string,"GL_SMOOTH_LINE_WIDTH_RANGE: %f %f (1.0 1.0)",ftab[0],ftab[1]);
    DrawString (0,height,string);
    height -= 16;

    // krok zmiany szerokoci linii z antyaliasingiem
    // OpenGL 1.2
    glGetFloatv (GL_SMOOTH_LINE_WIDTH_GRANULARITY,&fval);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_SMOOTH_LINE_WIDTH_GRANULARITY: %f",fval);
        DrawString (0,height,string);
    }
}

// wywietlenie drugiej strony ze zmiennym stanu zalenymi od implementacji

void Page2 ()
{
    // cig znakw uywany przy wywietlaniu napisw
    char string [256];

    // pobierane wartoci
    GLint ivar,itab [64];
    //GLboolean bval;
    //GLfloat fval,ftab [2];

    // wysoko okna renderingu
    GLuint height = glutGet (GLUT_WINDOW_HEIGHT) - 15;

    // wskanik na opcjonalan funkcj glGetConvolutionParameteriv
    PFNGLGETCONVOLUTIONPARAMETERIVPROC glGetConvolutionParameteriv = NULL;

    // pobranie wskanika na opcjonaln funkcj glGetConvolutionParameteriv
    glGetConvolutionParameteriv =
        (PFNGLGETCONVOLUTIONPARAMETERIVPROC) wglGetProcAddress ("glGetConvolutionParameteriv");

    // wywietlenie dwch opcjonalnych zmiennych stanu
    if (glGetConvolutionParameteriv != NULL)
    {
        // maksymalna szeroko filtra splotowego
        glGetConvolutionParameteriv (GL_CONVOLUTION_1D,GL_MAX_CONVOLUTION_WIDTH,itab+0);
        glGetConvolutionParameteriv (GL_CONVOLUTION_2D,GL_MAX_CONVOLUTION_WIDTH,itab+1);
        glGetConvolutionParameteriv (GL_SEPARABLE_2D,GL_MAX_CONVOLUTION_WIDTH,itab+2);
        sprintf (string,"GL_MAX_CONVOLUTION_WIDTH: %i %i %i (3 3 3)",itab[0],itab[1],itab[2]);
        DrawString (0,height,string);
        height -= 16;

        // maksymalna wysoko filtra splotowego
        glGetConvolutionParameteriv (GL_CONVOLUTION_2D,GL_MAX_CONVOLUTION_HEIGHT,itab+0);
        glGetConvolutionParameteriv (GL_SEPARABLE_2D,GL_MAX_CONVOLUTION_HEIGHT,itab+1);
        sprintf (string,"GL_MAX_CONVOLUTION_HEIGHT: %i %i (3 3)",itab[0],itab[1]);
        DrawString (0,height,string);
        height -= 16;
    }

    // rekomendowana maksymalna ilo indeksw tablic
    // wierzchokw (funkcja glDrawRangeElements)
    // OpenGL 1.2, EXT_draw_range_elements
    glGetIntegerv (GL_MAX_ELEMENTS_INDICES,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_ELEMENTS_INDICES: %i",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // rekomendowana maksymalna ilo elementw tablic
    // wierzchokw (funkcja glDrawRangeElements)
    // OpenGL 1.2, EXT_draw_range_elements
    glGetIntegerv (GL_MAX_ELEMENTS_VERTICES,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_ELEMENTS_VERTICES: %i",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // ilo buforw wieloprbkowania
    // OpenGL 1.3, ARB_multisample, SGIS_multisample, EXT_multisample
    glGetIntegerv (GL_SAMPLE_BUFFERS,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_SAMPLE_BUFFERS: %i (0)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // wielko maski pokrycia w wieloprbkowaniu
    // OpenGL 1.3, ARB_multisample, SGIS_multisample, EXT_multisample
    glGetIntegerv (GL_SAMPLES,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_SAMPLES: %i (0)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // obsugiwane formaty kompresji tekstur
    // OpenGL 1.3, ARB_texture_compression
    glGetIntegerv (GL_COMPRESSED_TEXTURE_FORMATS,itab);
    if (glGetError () == GL_NO_ERROR)
    {
        glGetIntegerv (GL_NUM_COMPRESSED_TEXTURE_FORMATS,&ivar);
        DrawString (0,height,"GL_COMPRESSED_TEXTURE_FORMATS:");
        height -= 16;
        for (int i = 0; i < ivar; i++)
        {
            switch (itab [i])
            {
                // formaty rozszerzenia EXT_texture_compression_s3tc
                case GL_COMPRESSED_RGB_S3TC_DXT1_EXT:
                sprintf (string,"GL_COMPRESSED_RGB_S3TC_DXT1_EXT");
                break;
                case GL_COMPRESSED_RGBA_S3TC_DXT1_EXT:
                sprintf (string,"GL_COMPRESSED_RGBA_S3TC_DXT1_EXT");
                break;
                case GL_COMPRESSED_RGBA_S3TC_DXT3_EXT:
                sprintf (string,"GL_COMPRESSED_RGBA_S3TC_DXT3_EXT");
                break;
                case GL_COMPRESSED_RGBA_S3TC_DXT5_EXT:
                sprintf (string,"GL_COMPRESSED_RGBA_S3TC_DXT5_EXT");
                break;

                // formaty rozszerzenia 3DFX_texture_compression_FXT1
                case GL_COMPRESSED_RGB_FXT1_3DFX:
                sprintf (string,"GL_COMPRESSED_RGB_FXT1_3DFX");
                break;
                case GL_COMPRESSED_RGBA_FXT1_3DFX:
                sprintf (string,"GL_COMPRESSED_RGBA_FXT1_3DFX");
                break;

                // format rozszerzenia ATI_texture_compression_3dc
                // (nie wystpuje w pliku glext.h)
                case 0x8837:
                sprintf (string,"GL_COMPRESSED_RGB_3DC_ATI");
                break;

                // formaty rozszerzenia EXT_texture_compression_latc
                case GL_COMPRESSED_LUMINANCE_LATC1_EXT:
                sprintf (string,"GL_COMPRESSED_LUMINANCE_LATC1_EXT");
                break;
                case GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT:
                sprintf (string,"GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT");
                break;
                case GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT:
                sprintf (string,"GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT");
                break;
                case GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT:
                sprintf (string,"GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT");
                break;

                // formaty rozszerzenia GL_EXT_texture_compression_rgtc
                case GL_COMPRESSED_RED_RGTC1_EXT:
                sprintf (string,"GL_COMPRESSED_RED_RGTC1_EXT");
                break;
                case GL_COMPRESSED_SIGNED_RED_RGTC1_EXT:
                sprintf (string,"GL_COMPRESSED_SIGNED_RED_RGTC1_EXT");
                break;
                case GL_COMPRESSED_RED_GREEN_RGTC2_EXT:
                sprintf (string,"GL_COMPRESSED_RED_GREEN_RGTC2_EXT");
                break;
                case GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT:
                sprintf (string,"GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT");
                break;

                // pozostae formaty
                default:
                sprintf (string,"Format nieznany (0x%X)",itab [i]);
                break;
            }
            DrawString (30,height,string);
            height -= 16;
        }
    }

    // ilo obsugiwanych formatw kompresji tekstur
    // OpenGL 1.3, ARB_texture_compression
    glGetIntegerv (GL_NUM_COMPRESSED_TEXTURE_FORMATS,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_NUM_COMPRESSED_TEXTURE_FORMATS: %i (0)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // wskanik na funkcj glGetQueryiv
    PFNGLGETQUERYIVPROC glGetQueryiv = NULL;

    // pobranie wskanika na funkcj glGetQueryiv
    glGetQueryiv = (PFNGLGETQUERYIVPROC) wglGetProcAddress ("glGetQueryiv");

    // wywietlenie liczby bitw licznika analizy przesoni
    // OpenGL 1.5, ARB_occlusion_query
    if (glGetQueryiv != NULL)
    {
        glGetQueryiv (GL_SAMPLES_PASSED,GL_QUERY_COUNTER_BITS,&ivar);

        // pobranie maksymalnej wielkoci obszaru renderingu
        glGetIntegerv (GL_MAX_VIEWPORT_DIMS,itab);

        // obliczenie minimalnej iloci bitw licznika analizy
        // przesnoni obugiwanych przez implementacj
        int dep_bits = (int)ceil (log ((double)2*itab[0]*itab[1]) / log (2.0));
        if (dep_bits > 32)
            dep_bits = 32;
        sprintf (string,"GL_QUERY_COUNTER_BITS: %i (%i)",ivar,dep_bits);
        DrawString (0,height,string);
        height -= 16;
    }

    // ilo jednostek teksturujych
    // OpenGL 1.3, ARB_multitexture
    glGetIntegerv (GL_MAX_TEXTURE_UNITS,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_TEXTURE_UNITS: %i (2)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalna ilo aktywnych atrybutw wierzchokw
    // OpenGL 2.0, ARB_vertex_program
    glGetIntegerv (GL_MAX_VERTEX_ATTRIBS,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_VERTEX_ATTRIBS: %i (16)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalna ilo zmiennych jednorodnych dla programw
    // cieniowania wierzchokw
    // OpenGL 2.0, ARB_vertex_shader
    glGetIntegerv (GL_MAX_VERTEX_UNIFORM_COMPONENTS,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_VERTEX_UNIFORM_COMPONENTS: %i (512)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalna ilo skadowych interpolowanych zmiennych przekazywanych
    // z programu cieniowania wierzchokw do programu cieniowania fragmentw
    // OpenGL 2.0, ARB_vertex_shader
    glGetIntegerv (GL_MAX_VARYING_FLOATS,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_VARYING_FLOATS: %i (32)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalna ilo programowalnych jednostek teksturujcych
    // OpenGL 2.0, ARB_vertex_shader
    glGetIntegerv (GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS: %i (2)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalna ilo programowalnych jednostek teksturujcych
    // dostpnych dla programw cieniowania wierzchokw
    // OpenGL 2.0, ARB_vertex_shader
    glGetIntegerv (GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS: %i (0)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalna ilo programowalnych jednostek teksturujcych
    // dostpnych dla programw cieniowania wierzchokw
    // OpenGL 2.0, ARB_fragment_program
    glGetIntegerv (GL_MAX_TEXTURE_IMAGE_UNITS,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_TEXTURE_IMAGE_UNITS: %i (2)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalna ilo zbiorw wsprzdnych tekstur
    // OpenGL 2.0, ARB_fragment_program
    glGetIntegerv (GL_MAX_TEXTURE_COORDS,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_TEXTURE_COORDS: %i (2)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalna ilo zmiennych jednorodnych dla
    // programw cieniowania wierzchokw
    // OpenGL 2.0, ARB_fragment_shader
    glGetIntegerv (GL_MAX_FRAGMENT_UNIFORM_COMPONENTS,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_FRAGMENT_UNIFORM_COMPONENTS: %i (64)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // maksymalna ilo aktywnych docelowych buforw koloru
    // OpenGL 2.0, ARB_draw_buffers
    glGetIntegerv (GL_MAX_DRAW_BUFFERS,&ivar);
    if (glGetError () == GL_NO_ERROR)
    {
        sprintf (string,"GL_MAX_DRAW_BUFFERS: %i (1+)",ivar);
        DrawString (0,height,string);
        height -= 16;
    }

    // ilo bitw skadowych RGBA (lub numerw indeksw) bufora kolorw
    // OpenGL 1.0
    glGetIntegerv (GL_RED_BITS,&ivar);
    sprintf (string,"GL_RED_BITS: %i (-)",ivar);
    DrawString (0,height,string);
    height -= 16;
    glGetIntegerv (GL_GREEN_BITS,&ivar);
    sprintf (string,"GL_GREEN_BITS: %i (-)",ivar);
    DrawString (0,height,string);
    height -= 16;
    glGetIntegerv (GL_BLUE_BITS,&ivar);
    sprintf (string,"GL_BLUE_BITS: %i (-)",ivar);
    DrawString (0,height,string);
    height -= 16;
    glGetIntegerv (GL_ALPHA_BITS,&ivar);
    sprintf (string,"GL_ALPHA_BITS: %i (-)",ivar);
    DrawString (0,height,string);
    height -= 16;
    glGetIntegerv (GL_INDEX_BITS,&ivar);
    sprintf (string,"GL_INDEX_BITS: %i (-)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // ilo bitw elementw bufora gbokoci
    // OpenGL 1.0
    glGetIntegerv (GL_DEPTH_BITS,&ivar);
    sprintf (string,"GL_DEPTH_BITS: %i (-)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // ilo bitw elementw bufora akumulacyjnego
    // OpenGL 1.0
    glGetIntegerv (GL_STENCIL_BITS,&ivar);
    sprintf (string,"GL_STENCIL_BITS: %i (-)",ivar);
    DrawString (0,height,string);
    height -= 16;

    // ilo bitw skadowych RGBA bufora akumulacyjnego
    // OpenGL 1.0
    glGetIntegerv (GL_ACCUM_RED_BITS,&ivar);
    sprintf (string,"GL_ACCUM_RED_BITS: %i (-)",ivar);
    DrawString (0,height,string);
    height -= 16;
    glGetIntegerv (GL_ACCUM_GREEN_BITS,&ivar);
    sprintf (string,"GL_ACCUM_GREEN_BITS: %i (-)",ivar);
    DrawString (0,height,string);
    height -= 16;
    glGetIntegerv (GL_ACCUM_BLUE_BITS,&ivar);
    sprintf (string,"GL_ACCUM_BLUE_BITS: %i (-)",ivar);
    DrawString (0,height,string);
    height -= 16;
    glGetIntegerv (GL_ACCUM_ALPHA_BITS,&ivar);
    sprintf (string,"GL_ACCUM_ALPHA_BITS: %i (-)",ivar);
    DrawString (0,height,string);
}

// wywietlenie trzeciej strony ze zmiennym stanu zalenymi od implementacji

void Page3 ()
{
    // cig znakw uywany przy wywietlaniu napisw
    char string [2048];

    // rozmiary okna renderingu
    GLuint height = glutGet (GLUT_WINDOW_HEIGHT) - 15;
    GLuint width = glutGet (GLUT_WINDOW_WIDTH);

    // autor implementacji OpenGL
    // OpenGL 1.0
    strcpy (string,"GL_VENDOR: ");
    strcat (string,(char*)glGetString (GL_VENDOR));
    DrawString (0,height,string);
    height -= 16;

    // nazwa urzdzenia renderujcego
    // OpenGL 1.0
    strcpy (string,"GL_RENDERER: ");
    strcat (string,(char*)glGetString (GL_RENDERER));
    DrawString (0,height,string);
    height -= 16;

    // numer wersji implementacji OpenGL
    // OpenGL 1.0
    strcpy (string,"GL_VERSION: ");
    strcat (string,(char*)glGetString (GL_VERSION));
    DrawString (0,height,string);
    height -= 16;

    // numer wersji obsugiwanego jzyka GLSL
    // OpenGL 2.0, ARB_shading_language_100
    strcpy (string,"GL_SHADING_LANGUAGE_VERSION: ");
    const GLubyte *str = glGetString (GL_SHADING_LANGUAGE_VERSION);
    if (str != NULL)
    {
        strcat (string,(char*)glGetString (GL_SHADING_LANGUAGE_VERSION));
        DrawString (0,height,string);
        height -= 16;
    }

    // lista obsugiwanych rozszerze OpenGL
    // OpenGL 1.0
    const GLubyte *extstr = glGetString (GL_EXTENSIONS);
    strcpy (string,"GL_EXTENSIONS: ");
    int extpos = 0;
    int extlen = strlen ((char*)extstr);
    while (extpos < extlen)
    {
        // selekcja nazwy rozszerzenia
        int pos = extpos;
        while (extstr [pos] != '\0' && extstr [pos] != ' ')
            pos++;

        // dodanie rozszerzenia do biecego cigu znakw
        if (9 * (pos - extpos + strlen (string)) < width)
        {
            strncat (string,(char*)&extstr [extpos],pos - extpos + 1);
            extpos = pos + 1;
        }

        // dodanie rozszerzenia do nastpnego cigu znakw
        else
        {
            DrawString (0,height,string);
            height -= 16;
            strcpy (string,"  ");
            strncat (string,(char*)&extstr [extpos],pos - extpos + 1);
            extpos = pos + 1;
        }

        // wywietlenie ostatniego cigu znakw
        if (extpos == extlen)
        {
            DrawString (0,height,string);
            height -= 16;
        }
    }
}

// funkcja generujca scen 3D

void DisplayScene ()
{
    // kolor ta - zawarto bufora koloru
    glClearColor (1.0,1.0,1.0,1.0);

    // czyszczenie bufora koloru
    glClear (GL_COLOR_BUFFER_BIT);

    // kolor napisw
    glColor3f (0.0,0.0,0.0);

    // wswietlenie wybranej strony
    switch (page_no)
    {
        case PAGE_1:
        Page1 ();
        break;
        case PAGE_2:
        Page2 ();
        break;
        case PAGE_3:
        Page3 ();
        break;
    }

    // skierowanie polece do wykonania
    glFlush ();

    // zamiana buforw koloru
    glutSwapBuffers ();
}

// zmiana wielkoci okna

void Reshape (int width, int height)
{
    // obszar renderingu - cae okno
    glViewport (0,0,width,height);

    // wybr macierzy rzutowania
    glMatrixMode (GL_PROJECTION);

    // macierz rzutowania = macierz jednostkowa
    glLoadIdentity ();

    // rzutowanie prostoktne
    gluOrtho2D (0,width,0,height);

    // generowanie sceny 3D
    DisplayScene ();
}

// obsuga menu podrcznego

void Menu (int value)
{
    switch (value)
    {
        case PAGE_1:
        case PAGE_2:
        case PAGE_3:
        page_no = value;
        DisplayScene ();
        break;

        // wyjcie
        case EXIT:
        exit (0);
    }
}

int main (int argc, char *argv[])
{
    // inicjalizacja biblioteki GLUT
    glutInit (&argc,argv);

    // inicjalizacja biblioteki GLUT
    glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_ACCUM | GLUT_DEPTH |
                         GLUT_STENCIL | GLUT_ALPHA);

    // rozmiary gwnego okna programu
    glutInitWindowSize (600,600);

    // rozmiary gwnego okna programu
    glutCreateWindow ("Implementacja OpenGL");

    // doczenie funkcji generujcej scen 3D
    glutDisplayFunc (DisplayScene);

    // doczenie funkcji wywoywanej przy zmianie rozmiaru okna
    glutReshapeFunc (Reshape);

    // menu gwne
    glutCreateMenu (Menu);
    glutAddMenuEntry ("Strona 1",PAGE_1);
    glutAddMenuEntry ("Strona 2",PAGE_2);
    glutAddMenuEntry ("Strona 3",PAGE_3);

#ifdef WIN32

    glutAddMenuEntry ("Wyjcie",EXIT);
#else

    glutAddMenuEntry ("Wyjscie",EXIT);
#endif

    // okrelenie przycisku myszki obsugujcego menu podrczne
    glutAttachMenu (GLUT_RIGHT_BUTTON);

    // wprowadzenie programu do obsugi ptli komunikatw
    glutMainLoop ();
    return 0;
}
